/*
 * bpamonitor.c -- monitor WAN interface status and dial or disconnect bpa connection.
 *
 *                                                                      -- Argon Cheng
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <net/if.h>
#include <syslog.h>
#include <signal.h>
#include "../nvram/nvram.h"
#include "timestamp.h"
#include "../trunk_include/wan.h"


/* same as kernel/include/linux/sockios.h */
#define SIOCGIFTTIMESTAMP   0x89F1
#define SIOCSIFTTIMESTAMP   0x89F2

//#define WAN_IF_NAME           "eth0"
#define CHECK_INTERVAL      1
enum {
    STATE_IDLE,
    STATE_WORKING
};
int state = STATE_IDLE;

#define _DEBUG_
#ifdef _DEBUG_
#define DBG_PRINTF(fmt, arg...)     syslog(LOG_INFO, "{%s():%d}"fmt, __FUNCTION__, __LINE__, ##arg)
#else
#define DBG_PRINTF(fmt, arg...)     printf("{%s():%d}"fmt, __FUNCTION__, __LINE__, ##arg)
#endif

int start_bpa() {
    DBG_PRINTF("begin in this func\n");
/*    char cmd[1024]="";
    if (strcmp(nvram_safe_get("wan_fix_dns"), "1") == 0)
    {
        sprintf(cmd,"/usr/sbin/udhcpc -i %s -s /etc/bpa/udhcpc.bpa.fix.script&"
               , nvram_safe_get("wan_ifname"));
    }
    else
    {
        sprintf(cmd,"/usr/sbin/udhcpc -i %s -s /etc/bpa/udhcpc.bpa.script&"
               , nvram_safe_get("wan_ifname"));
    }*/
    system("/usr/sbin/bpalogin -c /var/bpalogin.conf &");
//		system(cmd);
    return 0;
}

int stop_bpa() {
    DBG_PRINTF("begin in this func\n");
		system("/usr/bin/killall -SIGTERM bpalogin");
//    system("/usr/bin/killall -9 bpalogin");
		sleep(1);
//    system("/usr/bin/killall -SIGUSR2 udhcpc");
//    sleep(1);
//    system("/usr/bin/killall  -9 udhcpc");
    nvram_set("bpa_login", "0");
//		system("/usr/sbin/rc wan stop");
//		system("/bin/rm /tmp/wan_uptime");
    return 0;
}

void sig_connect(int sig) {
    DBG_PRINTF("current state is <%s>\n", state == STATE_WORKING?"STATE_WORKING":"STATE_IDLE");
    if(state != STATE_WORKING) {
        start_bpa();
        state = STATE_WORKING;
    }
    return;
}

void sig_disconnect(int sig) {
    DBG_PRINTF("current state is <%s>\n", state == STATE_WORKING?"STATE_WORKING":"STATE_IDLE");
    if(state != STATE_IDLE) {
        stop_bpa();
        state = STATE_IDLE;
    }
    return;
}

int main(int argc, char *argv[]) {
    unsigned long old_timestamp=0, cur_timestamp;
    int fd = get_sockfd();
    int max_idle_time = atoi(nvram_safe_get("bpa_idle")) * 60;
    int idle;
    int bpa_dod = atoi(nvram_safe_get("bpa_dod"));
    DBG_PRINTF("begin in bpamonitor bpa_dod is <%d> max_idle_time is <%d>s\n", bpa_dod, max_idle_time);


    signal(SIGUSR1, sig_connect);
    signal(SIGUSR2, sig_disconnect);

    if(fd < 0) {
        return -1;
    }

    /* if not dod mode, call bpalogin immediately */
    if(!bpa_dod) {
        start_bpa();
				close(fd);
        return 0;
    }

    while(1) {
        switch(state) {
        case STATE_IDLE:
            old_timestamp = get_tx_timestamp(fd, WAN_IF_NAME);
            DBG_PRINTF("old_timestamp is %lu\n", old_timestamp);
            while(1) {
                if(state != STATE_IDLE) {
                    /* state can be changed by signal, we need check it */
                    break;
                }                
                cur_timestamp = get_tx_timestamp(fd, WAN_IF_NAME);
                DBG_PRINTF("cur_timestamp is <%lu> old_timestamp is <%lu>\n", cur_timestamp, old_timestamp);
 //               printf(" idle cur_timestamp is <%lu> old_timestamp is <%lu>\n", cur_timestamp, old_timestamp);
                if(cur_timestamp != old_timestamp) {
                    /* WAN send packet! active bpa */
                    DBG_PRINTF("WAN send packet! active bpa\n");
                    start_bpa();
                    state = STATE_WORKING;
                    break;
                }
                sleep(CHECK_INTERVAL);
            }
            break;
        case STATE_WORKING:
            old_timestamp = get_tx_timestamp(fd, WAN_IF_NAME);
            idle = 0;
            while(1) {
                if(state != STATE_WORKING) {
                    /* state can be changed by signal, we need check it */
                    break;
                }
                cur_timestamp = get_tx_timestamp(fd, WAN_IF_NAME);
                DBG_PRINTF("cur_timestamp is <%lu> old_timestamp is <%lu>\n", cur_timestamp, old_timestamp);
               // printf("working cur_timestamp is <%lu> old_timestamp is <%lu>\n", cur_timestamp, old_timestamp);
                if(cur_timestamp != old_timestamp) {
                    /* WAN send packet, record the send time and reset idle */
                    old_timestamp = cur_timestamp;
                    idle = 0;
                }
                else {
                    idle++;
                    DBG_PRINTF("idle <%d> \n", idle);
                    if(idle > max_idle_time) {
                        /* idle ... disconnect bpa */
                        DBG_PRINTF("idle %d seconds ... disconnect bpa\n", idle);
                        stop_bpa();
                        state = STATE_IDLE;
                        break;
                    }
                }
                sleep(CHECK_INTERVAL);
            }
            break;
        default:
            break;
        }
    }

    close(fd);
    return 0;
}

