/*
modified by robinhood_eliu for forking. robin.hood@bigfoot.com

ras - Copyright (C) 2000 - Tan Thor Jen 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

Acknowledgements - Ivo Clarysse - soggie@soti.org 
*/
/*  SERCOMM RAS implement -- 28/4/2007  -- PBS  */
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include "../nvram/nvram.h"
#include "../trunk_include/wan.h"
typedef unsigned char byte;
typedef unsigned dword;
typedef unsigned short word;

int sockfd;
struct sockaddr_in gw_addr;

char userid[64] = "";
char password[64] = "";
char domain[64] = "INT512";
char interface[64] = WAN_IF_NAME;//"eth2.2";
char local_ip[64] = "";
char gw_ip[64] = "";

const char *dhcpcd_path = "/usr/sbin/udhcpc";
const char *pidfile = "/tmp/ras.pid";

dword client_counter = 0;
dword session_id;

int debug_level = 0;

/*
    TODO: 
*/

typedef struct __pack_buf__
{
    int length;
    byte buffer[1024];
}
Pack;

void
pack_init (Pack * buf)
{
    buf->length = 0;
}

void
pack_dword (Pack * buf, unsigned dword)
{
    *((unsigned *) (&buf->buffer[buf->length])) = dword;
    buf->length += 4;
}

void
packn_dword (Pack * buf, unsigned dword)
{
    pack_dword (buf, htonl (dword));
}

void
pack_w (Pack * buf, unsigned short w)
{
    *((unsigned short *) (&buf->buffer[buf->length])) = w;
    buf->length += 2;
}

void
packn_w (Pack * buf, unsigned w)
{
    pack_w (buf, htons (w));
}

void
pack_v (Pack * buf, byte * bytes, int bytes_len)
{
    int i;
    for (i = 0; i < bytes_len; i++)
    {
        buf->buffer[buf->length++] = bytes[i];
    }
}

void
pack_str (Pack * buf, const char *str)
{
    const char *p = str;
    while (*p != 0)
    {
        buf->buffer[buf->length++] = *p;
        p++;
    }
}

/* FIXME: ugly code */
void
dump_pack (Pack * buf)
{
    int i, j, k, ch;

    j = 0;

    for (i = 0; i < buf->length; i++)
    {
        if (i % 16 == 0 && i != 0)
        {
            printf (" ");
            for (k = j; k < i; k++)
            {
                ch = buf->buffer[k];
                printf ("%c", ch >= 32 ? ch : '.');
            }
            j = i;
            printf ("\n");
        }
        printf ("%.2X ", buf->buffer[i]);
    }

    for (k = i; k % 16 != 0; k++)
        printf (" ");
    printf (" ");

    for (k = j; k < i; k++)
    {
        ch = buf->buffer[k];
        printf ("%c", ch >= 32 ? ch : '.');
    }

    printf ("\nlen=%d\n\n", buf->length);
}

int
die_if (int boolean, const char *msg)
{
    if (boolean)
    {
        printf ("%s\n", msg);
        exit (1);
    }
}

int
send_pack (Pack * pack)
{
    int send_return;

    if (debug_level >= 10)
    {
        printf ("Sending.\n");
        dump_pack (pack);
    }

    send_return =
    sendto (sockfd, &pack->buffer, pack->length, 0, &gw_addr,
    sizeof (gw_addr));

    // sleep(1); // FIXME: is this needed to slow down transmissions?

    return send_return == -1 ? 0 : 1;
}

int
send_ack (Pack * pack)
{
    byte unk[6] = { 0, 0, 0, 0, 0, 0 };

    pack->length = 12;
    pack->buffer[1]++;
    pack->buffer[2] = 0;
    pack->buffer[3] = 0;
    // pack_v(pack, unk, 6);
    // apparently not needed to fillout buffer

    return send_pack (pack);
}

int
send_open_session_req (void)
{
    Pack pack;

    // apparently not needed
    byte unknown[] = {
        0x00, 0x05, 0x01, 0x00,
        0x05, 0x00, 0x09, 0xC2,
        0x23, 0x05, 0xC0, 0x23,
        0x00, 0x04, 0x00, 0x08,
        0x65, 0x6E, 0x75, 0x6B,
        0x00, 0x03, 0x00, 0x06,
        0x03, 0x00, 0x00, 0x02
    };

    pack_init (&pack);
    packn_dword (&pack, 0x01010000);
    packn_dword (&pack, client_counter++);
    packn_dword (&pack, 0);
    packn_w (&pack, 1);
    packn_w (&pack, 0x2);
    // pack_v(&pack, unknown, sizeof(unknown));
    packn_w (&pack, 0x0c);
    pack_str (&pack, "Rev. 1.1");
    packn_w (&pack, 1);
    packn_w (&pack, 0x13);
    pack_str (&pack, "Alcatel Telecom");
    send_pack (&pack);
}

int
send_add_call (void)
{
    Pack pack;

    pack_init (&pack);
    packn_dword (&pack, 0x01090000);
    packn_dword (&pack, client_counter++);
    packn_dword (&pack, session_id);

    packn_w (&pack, 7);
    packn_w (&pack, strlen (password) + 6);
    packn_w (&pack, 3);
    pack_str (&pack, password);

    packn_w (&pack, 8);
    packn_w (&pack, strlen (domain) + 6);
    packn_w (&pack, 3);
    pack_str (&pack, domain);

    packn_w (&pack, 6);
    packn_w (&pack, strlen (userid) + 6);
    packn_w (&pack, 3);
    pack_str (&pack, userid);

    return send_pack (&pack);
}

int
send_close_session(void)
{
    Pack pack;


   	nvram_set("ras_login", "0");
    pack_init (&pack);
    packn_dword (&pack, 0x01050000);
    packn_dword (&pack, client_counter++);
    packn_dword (&pack, session_id);
//    return send_pack (&pack);
    send_pack (&pack);
		exit(1);
}

int
send_simple (dword msg_id)
{
    Pack pack;

    pack_init (&pack);
    packn_dword (&pack, msg_id);
    packn_dword (&pack, client_counter++);
    packn_dword (&pack, session_id);

    return send_pack (&pack);
}

int
recv_pack (Pack * pack)
{
    fd_set rfds;
    struct timeval tv;
    int retval;

    int recv_return;

    FD_ZERO (&rfds);
    FD_SET (sockfd, &rfds);
    tv.tv_sec = 20;
    tv.tv_usec = 0;

    retval = select (sockfd + 1, &rfds, NULL, NULL, &tv);

    if (retval > 0)
    {
        pack_init (pack);
        recv_return = recv (sockfd, &pack->buffer, sizeof (pack->buffer), 0);
        die_if (recv_return == -1, "recv() failed");
        pack->length = recv_return;

        if (debug_level >= 10)
        {
            printf ("Received.\n");
            dump_pack (pack);
        }
        return 1;
    }
    else if (retval == 0)
    {
        return 0;
    }

    die_if (1, "select() failed");
}

void
parse_dhcpcd_files (void)
{
    FILE *f;
    char buf[256];
		int count =0;
    sprintf (buf, "/tmp/dhcpc.lease");
 /*   f = fopen (buf, "r");
    die_if (f == NULL, "Unable to open dhcpcd files");
*/
		while((f=fopen(buf,"r"))==NULL)
		{
			count ++;
			printf("unable to open dhcpc file . waiting ...%d\n",count);
			if(count<150)
			sleep(2); //if wan can not get dhcp IP , we will wait at most 5 minutes .
			else
      exit (1);			
			
			}
    if(fgets (buf, sizeof (buf), f) != NULL)
    {
        sscanf (buf, "%s", local_ip);
//        sscanf (buf, "GATEWAY=%s", gw_ip);
    }
    fclose (f);

		sprintf(buf,"/tmp/wan_dhcp_ras");
		f = fopen(buf,"r");
		die_if(f == NULL, "Unable to open dhcpc files");
		
    if(fgets (buf, sizeof (buf), f) != NULL)
    {
//        sscanf (buf, "IPADDR=%s", local_ip);
        sscanf (buf, "%s", gw_ip);
    }
    fclose (f);


    if (debug_level >= 5)
    {
        printf ("IP=%s\nGW=%s\n", local_ip, gw_ip);
    }
}

void
init_socket ()
{
    struct sockaddr_in addr;
    int bind_return;

    sockfd = socket (PF_INET, SOCK_DGRAM, 0);
    die_if (sockfd == -1, "socket failed");

    addr.sin_family = AF_INET;
//    die_if (inet_aton (local_ip, &addr.sin_addr) == 0, "inet_aton() failed");
//		my_addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = htonl(INADDR_ANY);

//		my_addr.sin_port = htons(5555);
    addr.sin_port = ntohs (5555);
    bind_return = bind (sockfd, &addr, sizeof (addr));
    die_if (bind_return == -1, "bind failed");


		printf("local bind successful ! \n");

// remote_addr.sin_family = AF_INET;
// remote_addr.sin_addr.s_addr = inet_addr(myspecs.server);
// remote_addr.sin_port =  htons(dana_port);
 
    gw_addr.sin_family = AF_INET;
//   die_if (inet_aton (gw_ip, &gw_addr.sin_addr) == 0, "inet_aton() failed");
		gw_addr.sin_addr.s_addr = inet_addr(gw_ip);
    gw_addr.sin_port = ntohs (5555);
}

word
get_pack_msg_id (Pack * pack)
{
    return ntohs (*((word *) (pack->buffer)));
}

int
expect_ack (word msg_id)
{
    Pack pack;
    if (!recv_pack (&pack))
        return 0;
    if (get_pack_msg_id (&pack) != msg_id)
        return 0;
    return 1;
}

int
expect_msg (word msg_id)
{
    Pack pack;

    if (!recv_pack (&pack))
        return 0;
    if (get_pack_msg_id (&pack) != msg_id)
        return 0;
    return send_ack (&pack);
}

void
start_dhcp ()
{
    char buf[256];
/*
    sprintf (buf, "%s -i %s -s /etc/udhcpc.ras.script&", dhcpcd_path, interface);
    system (buf);
    sleep (1); // ensure dhcpcd dies (somehow it forks itself before returning/killing itself i think)
*/
		system("/usr/bin/killall udhcpc &");
		sleep(2);
    sprintf (buf, "%s -i %s -s /etc/udhcpc.ras.script&", dhcpcd_path, interface);
    while (system (buf) != 0)
    {
        sleep (6);
    }
    parse_dhcpcd_files ();
}

// FIXME: really poor coding 
// FIXME: handle recv_pack timeouts
// returns 1 if logged in, else 0 to redo
int
do_login ()
{
    Pack pack;
    int success = 0;

    start_dhcp ();
		sleep(2);
    init_socket ();

    if (!send_open_session_req ())
    	{
    		printf("debug open session failed .... \n");
        return 0;
      }
    if (!expect_ack (0x0102))
    	{
        return 0;
      }
    if (!recv_pack (&pack))
    	{
        return 0;
      }
    if (get_pack_msg_id (&pack) != 0x0103)
    	{
        return 0;
      }
    session_id = ntohl (*((dword *) (&pack.buffer[8])));
    if (!send_ack (&pack))
    	{
        return 0;
      }

    if (!send_add_call ())
        return 0;
    if (!expect_ack (0x010A))
        return 0; // ack add_call

    if (!recv_pack (&pack))
        return 0; // add_call_ind
    if (pack.length >= 37)
        success = 1;
    if (!send_ack (&pack))
        return 0;

    if (success)
        return 1;

// otherwise, prepare to switch IPs
    if (!expect_msg (0x0117))
        return 0; // ip_switch
    if (!send_simple (0x01190000))
        return 0; // ip_switch_cnf;
    if (!expect_ack (0x011A))
        return 0; // ack ip_switch_cnf


		close(sockfd);
		sleep(2);
    start_dhcp ();
    init_socket ();

    if (!send_simple (0x011B0000))
        return 0; // continue_session_req
    if (!expect_ack (0x011C))
        return 0; // ack continue_session_req
    if (!expect_msg (0x011D))
        return 0; // continue_session_cnf
    if (!expect_msg (0x010B))
        return 0; // add_call_ind
    if (!expect_msg (0x010B))
        return 0; // add_call_ind
		nvram_set("ras_login", "1");
		system("/bin/cp /proc/uptime /tmp/wan_uptime");
    return 1;
}

int
main (int argc, char *argv[])
{

    FILE *fpid = NULL;
    int i, pid;

    signal(SIGUSR2, send_close_session); //add a aignal control to handle disconnet 

    while (1)
    {
        int c = getopt (argc, argv, "u:p:i:d:g::");
        if (c == -1)
            break;
        switch (c)
        {
            case 'u':
                strcpy (userid, optarg);
                break;
            case 'p':
                strcpy (password, optarg);
                break;
            case 'd':
                strcpy (domain, optarg);
                break;
            case 'i':
                strcpy (interface, optarg);
                break;
            case 'g':
                if (optarg == NULL)
                {
                    debug_level = 99;
                }
                else
                {
                    debug_level = atoi (optarg);
                }
                break;
            default:
                die_if (1, "getopt() error");
        }
    }

    // ensure saneness
    die_if (strcmp (userid, "") == 0, "No userid (-u) specified");
    die_if (strcmp (password, "") == 0, "No password (-p) specified");

    while (1)
    {
        for ( i = 0; i < 5; i++ )
	{
	    if (!do_login ())
	    {
        	printf ("the %d Login failed. Trying again...\n",i);
        	sleep (3); //wait a while before trying again
        	nvram_set("ras_login", "0");
	    } else
		goto enter_daemon;
        }

	puts( "Login failed. Please check your login token." );
	exit( 2 );

enter_daemon:
	// entering daemon mode.

	// notice, according to the loop, we fork everytime we establish
 	// the connection, that means even though (in a forked daemon)
	// we somehow get disconnected, the daemon will attempt to
	// reconnect and then fork another daemon. the old parent process
	// will exit LEAVING ONLY ONE COPY OF RAS.C in memory.
	
	printf("radebug : ras login successful \n");
	//restart wan service 
	system("/usr/sbin/rc service restart &");
	pid = fork();

        switch (pid)
        {
	    case 0:     // forked child is now running.
		break;
	    case -1:
		fputs( "Error while forking. Disconnecting.", stderr );
		start_dhcp();		// release dhcp lease.
		exit( 2 );
		break;
	    default:
		if ( (fpid = fopen( pidfile, "w" )) == NULL ) {
		    fputs( "Error writing pidfile. "     
			"Connection will continue.", stderr );
		    fprintf( stderr, "PID: %d\n", pid );
		    exit( 2 );
		}
		fprintf( fpid, "%d\n", pid );
		fclose( fpid );
		puts( "Connection Established." );
		exit( 0 );
		break;
	}

        while (1)
        {
            Pack pack;
            if (!send_simple (0x01070000))
                break; //send_hello
            if (!expect_ack (0x0108))
                break;
            sleep (20);		// changed for better safety.
        }
				nvram_set("ras_login", "0");
        puts ("Disconnected. Retrying ...");
    }
    close(sockfd);
    return 0;
}
